window.Gleam.initSearch([{"doc":"for_the_crows","title":"for_the_crows","content":"# crow\n\n[![Package Version](https://img.shields.io/hexpm/v/crow)](https://hex.pm/packages/crow)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/crow/)\n\nA Gleam project\n\n## Quick start\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\nIf available on Hex this package can be added to your Gleam project:\n\n```sh\ngleam add crow\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/crow&gt;.\n\n## Presentation \n\nPRESENTATION - Gleam is expressive\n\nGleam is fairly simple and in some ways syntax is less expressive than what\ncan be found in erlang or elixir: \n\n* Lack of literals\n* Lack of branching options\n* Lack of macro system\n* No behaviour or protocol system\n\nBut is just as capable of modelling most problem domains:\n\n* &quot;Open&quot; domain through types public types\n* Constained set of operations through opaque types\n  * Api might be &quot;flexible&quot;, next = gamestate -&gt; next -&gt; gamestate\n  * Api might be &quot;precise&quot;, next = gamestate -&gt; gamestate\n\nMay or not require assertion checks when depending on Result types\n  * Result types can be composed together\n  * But composition can also be achieved with pipes\n  * Or a custom mechanism to apply types or functions\n\nWe may add to this type through Generics.\n\nTODO: Recover gamestate file here\n\npub type MoveError {\n  NoPath\n  OutOfBounds\n  Blocked\n  NoPiece\n}\n\npub fn move(\n  in board: Board,\n  from from: Position,\n  to to: Position,\n) -&gt; Result(Board, MoveError) {\n  try Nil = check_bounds(at: from, in: board)\n  try Nil = check_bounds(at: to, in: board)\n  try Nil = check_piece(at: from, in: board)\n  try Nil = check_move(from: from, to: to, in: board)\n  Ok(update(at: from, to: to, in: board))\n}\n","url":"index.html"},{"doc":"crow/coordinate","title":"Coordinate","content":"pub type Coordinate {\n  Coordinate(x: Int, y: Int)\n}\n\nCoordinate(x: Int, y: Int)\n\n\nSynonyms:\ncrow/coordinate.Coordinate\ncrow/coordinate Coordinate","url":"crow/coordinate.html#Coordinate"},{"doc":"crow/coordinate","title":"add","content":"pub fn add(m: Coordinate, n: Coordinate) -&gt; Coordinate\n\nSynonyms:\ncrow/coordinate.add\ncrow/coordinate add","url":"crow/coordinate.html#add"},{"doc":"crow/coordinate","title":"equal","content":"pub fn equal(coordinate_a: Coordinate, coordinate_b: Coordinate) -&gt; Bool\n\nSynonyms:\ncrow/coordinate.equal\ncrow/coordinate equal","url":"crow/coordinate.html#equal"},{"doc":"crow/coordinate","title":"multiply","content":"pub fn multiply(m: Coordinate, n: Coordinate) -&gt; Coordinate\n\nSynonyms:\ncrow/coordinate.multiply\ncrow/coordinate multiply","url":"crow/coordinate.html#multiply"},{"doc":"crow/coordinate","title":"crow/coordinate","content":"","url":"crow/coordinate.html"},{"doc":"crow/grid","title":"Error","content":"pub type Error {\n  NegativeInitialization\n  OutOfBounds\n  NoContent\n}\n\nNegativeInitialization\n\n\nOutOfBounds\n\n\nNoContent\n\n\nSynonyms:\ncrow/grid.Error\ncrow/grid Error","url":"crow/grid.html#Error"},{"doc":"crow/grid","title":"Grid","content":"pub type Grid(content) {\n  Grid(limits: Limits, positions: Map(Coordinate, content))\n}\n\nGrid(limits: Limits, positions: Map(Coordinate, content))\n\n\nSynonyms:\ncrow/grid.Grid\ncrow/grid Grid","url":"crow/grid.html#Grid"},{"doc":"crow/grid","title":"Limits","content":"pub type Limits {\n  Limits(x: Coordinate, y: Coordinate)\n}\n\nLimits(x: Coordinate, y: Coordinate)\n\n\nSynonyms:\ncrow/grid.Limits\ncrow/grid Limits","url":"crow/grid.html#Limits"},{"doc":"crow/grid","title":"delete","content":"pub fn delete(in grid: Grid(a), at position: Coordinate) -&gt; Grid(\n  a,\n)\n\nSynonyms:\ncrow/grid.delete\ncrow/grid delete","url":"crow/grid.html#delete"},{"doc":"crow/grid","title":"filter","content":"pub fn filter(grid: Grid(a), apply: fn(Coordinate, a) -&gt; Bool) -&gt; Grid(\n  a,\n)\n\nSynonyms:\ncrow/grid.filter\ncrow/grid filter","url":"crow/grid.html#filter"},{"doc":"crow/grid","title":"from_list","content":"pub fn from_list(grid: Grid(a), list: List(#(Coordinate, a))) -&gt; Grid(\n  a,\n)\n\nSynonyms:\ncrow/grid.from_list\ncrow/grid from_list","url":"crow/grid.html#from_list"},{"doc":"crow/grid","title":"in_bounds","content":"pub fn in_bounds(in grid: Grid(a), at position: Coordinate) -&gt; Result(\n  Coordinate,\n  Error,\n)\n\nSynonyms:\ncrow/grid.in_bounds\ncrow/grid in_bounds","url":"crow/grid.html#in_bounds"},{"doc":"crow/grid","title":"map","content":"pub fn map(grid: Grid(a), apply: fn(Coordinate, a) -&gt; a) -&gt; Grid(\n  a,\n)\n\nSynonyms:\ncrow/grid.map\ncrow/grid map","url":"crow/grid.html#map"},{"doc":"crow/grid","title":"new","content":"pub fn new(x: Int, y: Int) -&gt; Result(Grid(a), Error)\n\nSynonyms:\ncrow/grid.new\ncrow/grid new","url":"crow/grid.html#new"},{"doc":"crow/grid","title":"retrieve","content":"pub fn retrieve(in grid: Grid(a), at position: Coordinate) -&gt; Result(\n  a,\n  Error,\n)\n\nSynonyms:\ncrow/grid.retrieve\ncrow/grid retrieve","url":"crow/grid.html#retrieve"},{"doc":"crow/grid","title":"set","content":"pub fn set(in grid: Grid(a), at position: Coordinate, value value: a) -&gt; Result(\n  Grid(a),\n  Error,\n)\n\nSynonyms:\ncrow/grid.set\ncrow/grid set","url":"crow/grid.html#set"},{"doc":"crow/grid","title":"update","content":"pub fn update(in grid: Grid(a), at from: Coordinate, to to: Coordinate) -&gt; Result(\n  Grid(a),\n  Error,\n)\n\nSynonyms:\ncrow/grid.update\ncrow/grid update","url":"crow/grid.html#update"},{"doc":"crow/grid","title":"crow/grid","content":"","url":"crow/grid.html"},{"doc":"crow/trace","title":"Trace","content":"pub opaque type Trace\n\n\nSynonyms:\ncrow/trace.Trace\ncrow/trace Trace","url":"crow/trace.html#Trace"},{"doc":"crow/trace","title":"get_from","content":"pub fn get_from(trace: Trace) -&gt; Coordinate\n\nSynonyms:\ncrow/trace.get_from\ncrow/trace get_from","url":"crow/trace.html#get_from"},{"doc":"crow/trace","title":"get_path","content":"pub fn get_path(trace: Trace) -&gt; List(Coordinate)\n\nSynonyms:\ncrow/trace.get_path\ncrow/trace get_path","url":"crow/trace.html#get_path"},{"doc":"crow/trace","title":"get_position","content":"pub fn get_position(trace: Trace) -&gt; Coordinate\n\nSynonyms:\ncrow/trace.get_position\ncrow/trace get_position","url":"crow/trace.html#get_position"},{"doc":"crow/trace","title":"get_step","content":"pub fn get_step(trace: Trace) -&gt; Int\n\nSynonyms:\ncrow/trace.get_step\ncrow/trace get_step","url":"crow/trace.html#get_step"},{"doc":"crow/trace","title":"get_trajectory","content":"pub fn get_trajectory(trace: Trace) -&gt; Coordinate\n\nSynonyms:\ncrow/trace.get_trajectory\ncrow/trace get_trajectory","url":"crow/trace.html#get_trajectory"},{"doc":"crow/trace","title":"new","content":"pub fn new(from: Coordinate, trajectory: Coordinate) -&gt; Trace\n\nSynonyms:\ncrow/trace.new\ncrow/trace new","url":"crow/trace.html#new"},{"doc":"crow/trace","title":"next","content":"pub fn next(trace: Trace) -&gt; Trace\n\nSynonyms:\ncrow/trace.next\ncrow/trace next","url":"crow/trace.html#next"},{"doc":"crow/trace","title":"crow/trace","content":"","url":"crow/trace.html"},{"doc":"crow/players","title":"Player","content":"pub type Player {\n  Player(id: String)\n}\n\nPlayer(id: String)\n\n\nSynonyms:\ncrow/players.Player\ncrow/players Player","url":"crow/players.html#Player"},{"doc":"crow/players","title":"current","content":"pub fn current(players: Queue(Player)) -&gt; Player\n\nSynonyms:\ncrow/players.current\ncrow/players current","url":"crow/players.html#current"},{"doc":"crow/players","title":"new","content":"pub fn new(players: List(Player)) -&gt; Queue(Player)\n\nSynonyms:\ncrow/players.new\ncrow/players new","url":"crow/players.html#new"},{"doc":"crow/players","title":"rotate","content":"pub fn rotate(players: Queue(Player)) -&gt; Queue(Player)\n\nSynonyms:\ncrow/players.rotate\ncrow/players rotate","url":"crow/players.html#rotate"},{"doc":"crow/players","title":"crow/players","content":"","url":"crow/players.html"},{"doc":"crow/piece","title":"Piece","content":"pub type Piece {\n  Pawn\n  Rook\n  Bishop\n  Knight\n  Queen\n  King\n}\n\nPawn\n\n\nRook\n\n\nBishop\n\n\nKnight\n\n\nQueen\n\n\nKing\n\n\nSynonyms:\ncrow/piece.Piece\ncrow/piece Piece","url":"crow/piece.html#Piece"},{"doc":"crow/piece","title":"crow/piece","content":"","url":"crow/piece.html"},{"doc":"crow/space","title":"Space","content":"pub type Space {\n  Space(\n    player: Player,\n    piece: Piece,\n    path: Set(Coordinate),\n    transform: Transform,\n  )\n}\n\nSpace(\n  player: Player,\n  piece: Piece,\n  path: Set(Coordinate),\n  transform: Transform,\n)\n\n\nSynonyms:\ncrow/space.Space\ncrow/space Space","url":"crow/space.html#Space"},{"doc":"crow/space","title":"Transform","content":"pub type Transform {\n  Normal\n  Inverse\n}\n\nNormal\n\n\nInverse\n\n\nSynonyms:\ncrow/space.Transform\ncrow/space Transform","url":"crow/space.html#Transform"},{"doc":"crow/space","title":"crow/space","content":"","url":"crow/space.html"},{"doc":"crow/rule","title":"Rule","content":"pub opaque type Rule(t, error)\n\n\nSynonyms:\ncrow/rule.Rule\ncrow/rule Rule","url":"crow/rule.html#Rule"},{"doc":"crow/rule","title":"apply","content":"pub fn apply(data: a, rule: Rule(a, b)) -&gt; Result(a, b)\n\nSynonyms:\ncrow/rule.apply\ncrow/rule apply","url":"crow/rule.html#apply"},{"doc":"crow/rule","title":"new","content":"pub fn new(rule: fn(a) -&gt; Result(a, b)) -&gt; Rule(a, b)\n\nSynonyms:\ncrow/rule.new\ncrow/rule new","url":"crow/rule.html#new"},{"doc":"crow/rule","title":"then","content":"pub fn then(rule: Rule(a, b), then_do: Rule(a, b)) -&gt; Rule(a, b)\n\nSynonyms:\ncrow/rule.then\ncrow/rule then","url":"crow/rule.html#then"},{"doc":"crow/rule","title":"crow/rule","content":"","url":"crow/rule.html"},{"doc":"crow/projection","title":"Projection","content":"pub type Projection(stop_reason) {\n  Projection(trace: Trace, reason: stop_reason)\n}\n\nProjection(trace: Trace, reason: stop_reason)\n\n\nSynonyms:\ncrow/projection.Projection\ncrow/projection Projection","url":"crow/projection.html#Projection"},{"doc":"crow/projection","title":"Stop","content":"pub opaque type Stop(reason)\n\n\nSynonyms:\ncrow/projection.Stop\ncrow/projection Stop","url":"crow/projection.html#Stop"},{"doc":"crow/projection","title":"continue","content":"pub fn continue(trace: Trace) -&gt; Result(Trace, Stop(a))\n\nSynonyms:\ncrow/projection.continue\ncrow/projection continue","url":"crow/projection.html#continue"},{"doc":"crow/projection","title":"next","content":"pub fn next(reason: a) -&gt; Result(Trace, Stop(a))\n\nSynonyms:\ncrow/projection.next\ncrow/projection next","url":"crow/projection.html#next"},{"doc":"crow/projection","title":"project","content":"pub fn project(from: Coordinate, trajectory: Coordinate, rule: Rule(\n    Trace,\n    Stop(a),\n  )) -&gt; Projection(a)\n\nSynonyms:\ncrow/projection.project\ncrow/projection project","url":"crow/projection.html#project"},{"doc":"crow/projection","title":"stop","content":"pub fn stop(reason: a) -&gt; Result(Trace, Stop(a))\n\nSynonyms:\ncrow/projection.stop\ncrow/projection stop","url":"crow/projection.html#stop"},{"doc":"crow/projection","title":"crow/projection","content":"","url":"crow/projection.html"},{"doc":"crow/direction","title":"bottom_left","content":"pub const bottom_left: Coordinate = Coordinate(-1, -1)\n\nSynonyms:\ncrow/direction.bottom_left\ncrow/direction bottom_left","url":"crow/direction.html#bottom_left"},{"doc":"crow/direction","title":"bottom_left_jump","content":"pub const bottom_left_jump: Coordinate = Coordinate(-1, -2)\n\nSynonyms:\ncrow/direction.bottom_left_jump\ncrow/direction bottom_left_jump","url":"crow/direction.html#bottom_left_jump"},{"doc":"crow/direction","title":"bottom_right","content":"pub const bottom_right: Coordinate = Coordinate(1, -1)\n\nSynonyms:\ncrow/direction.bottom_right\ncrow/direction bottom_right","url":"crow/direction.html#bottom_right"},{"doc":"crow/direction","title":"bottom_right_jump","content":"pub const bottom_right_jump: Coordinate = Coordinate(1, -2)\n\nSynonyms:\ncrow/direction.bottom_right_jump\ncrow/direction bottom_right_jump","url":"crow/direction.html#bottom_right_jump"},{"doc":"crow/direction","title":"down","content":"pub const down: Coordinate = Coordinate(0, -1)\n\nSynonyms:\ncrow/direction.down\ncrow/direction down","url":"crow/direction.html#down"},{"doc":"crow/direction","title":"hold","content":"pub const hold: Coordinate = Coordinate(0, 0)\n\nSynonyms:\ncrow/direction.hold\ncrow/direction hold","url":"crow/direction.html#hold"},{"doc":"crow/direction","title":"left","content":"pub const left: Coordinate = Coordinate(-1, 0)\n\nSynonyms:\ncrow/direction.left\ncrow/direction left","url":"crow/direction.html#left"},{"doc":"crow/direction","title":"left_bottom_jump","content":"pub const left_bottom_jump: Coordinate = Coordinate(-2, -1)\n\nSynonyms:\ncrow/direction.left_bottom_jump\ncrow/direction left_bottom_jump","url":"crow/direction.html#left_bottom_jump"},{"doc":"crow/direction","title":"left_top_jump","content":"pub const left_top_jump: Coordinate = Coordinate(-2, 1)\n\nSynonyms:\ncrow/direction.left_top_jump\ncrow/direction left_top_jump","url":"crow/direction.html#left_top_jump"},{"doc":"crow/direction","title":"right","content":"pub const right: Coordinate = Coordinate(1, 0)\n\nSynonyms:\ncrow/direction.right\ncrow/direction right","url":"crow/direction.html#right"},{"doc":"crow/direction","title":"right_top_jump","content":"pub const right_top_jump: Coordinate = Coordinate(2, 1)\n\nSynonyms:\ncrow/direction.right_top_jump\ncrow/direction right_top_jump","url":"crow/direction.html#right_top_jump"},{"doc":"crow/direction","title":"top_bottom_jump","content":"pub const top_bottom_jump: Coordinate = Coordinate(2, -1)\n\nSynonyms:\ncrow/direction.top_bottom_jump\ncrow/direction top_bottom_jump","url":"crow/direction.html#top_bottom_jump"},{"doc":"crow/direction","title":"top_left","content":"pub const top_left: Coordinate = Coordinate(-1, 1)\n\nSynonyms:\ncrow/direction.top_left\ncrow/direction top_left","url":"crow/direction.html#top_left"},{"doc":"crow/direction","title":"top_left_jump","content":"pub const top_left_jump: Coordinate = Coordinate(-1, 2)\n\nSynonyms:\ncrow/direction.top_left_jump\ncrow/direction top_left_jump","url":"crow/direction.html#top_left_jump"},{"doc":"crow/direction","title":"top_right","content":"pub const top_right: Coordinate = Coordinate(1, 1)\n\nSynonyms:\ncrow/direction.top_right\ncrow/direction top_right","url":"crow/direction.html#top_right"},{"doc":"crow/direction","title":"top_right_jump","content":"pub const top_right_jump: Coordinate = Coordinate(1, 2)\n\nSynonyms:\ncrow/direction.top_right_jump\ncrow/direction top_right_jump","url":"crow/direction.html#top_right_jump"},{"doc":"crow/direction","title":"up","content":"pub const up: Coordinate = Coordinate(0, 1)\n\nSynonyms:\ncrow/direction.up\ncrow/direction up","url":"crow/direction.html#up"},{"doc":"crow/direction","title":"crow/direction","content":"","url":"crow/direction.html"},{"doc":"crow/move","title":"Blocked","content":"pub type Blocked {\n  FriendlyPiece\n  RivalPiece\n  OutOfBounds\n  StepLimit\n  Capture\n}\n\nFriendlyPiece\n\n\nRivalPiece\n\n\nOutOfBounds\n\n\nStepLimit\n\n\nCapture\n\n\nSynonyms:\ncrow/move.Blocked\ncrow/move Blocked","url":"crow/move.html#Blocked"},{"doc":"crow/move","title":"MoveLimit","content":"pub type MoveLimit {\n  Continuous\n  Limited(n: Int)\n}\n\nContinuous\n\n\nLimited(n: Int)\n\n\nSynonyms:\ncrow/move.MoveLimit\ncrow/move MoveLimit","url":"crow/move.html#MoveLimit"},{"doc":"crow/move","title":"project","content":"pub fn project(board: Grid(Space), from: Coordinate) -&gt; Set(\n  Coordinate,\n)\n\nSynonyms:\ncrow/move.project\ncrow/move project","url":"crow/move.html#project"},{"doc":"crow/move","title":"crow/move","content":"","url":"crow/move.html"},{"doc":"chess","title":"GameState","content":"pub type GameState {\n  GameState(\n    players: Queue(Player),\n    board: Grid(Space),\n    stage: Stage,\n    turn: Int,\n    check: Map(Player, List(Coordinate)),\n    mate: Option(Player),\n    message: String,\n  )\n}\n\nGameState(\n  players: Queue(Player),\n  board: Grid(Space),\n  stage: Stage,\n  turn: Int,\n  check: Map(Player, List(Coordinate)),\n  mate: Option(Player),\n  message: String,\n)\n\n\nSynonyms:\nchess.GameState\nchess GameState","url":"chess.html#GameState"},{"doc":"chess","title":"MoveError","content":"pub type MoveError {\n  NoPath\n  EmptySpace\n}\n\nNoPath\n\n\nEmptySpace\n\n\nSynonyms:\nchess.MoveError\nchess MoveError","url":"chess.html#MoveError"},{"doc":"chess","title":"Stage","content":"pub type Stage {\n  Setup\n  Playing\n  End\n}\n\nSetup\n\n\nPlaying\n\n\nEnd\n\n\nSynonyms:\nchess.Stage\nchess Stage","url":"chess.html#Stage"},{"doc":"chess","title":"next","content":"pub fn next(state: GameState, from from: Coordinate, to to: Coordinate) -&gt; GameState\n\nSynonyms:\nchess.next\nchess next","url":"chess.html#next"},{"doc":"chess","title":"set","content":"pub fn set(board: Grid(Space), x: Int, y: Int, player: Player, piece: Piece, transform: Transform) -&gt; Result(\n  Grid(Space),\n  Error,\n)\n\nSynonyms:\nchess.set\nchess set","url":"chess.html#set"},{"doc":"chess","title":"setup","content":"pub fn setup(p1: String, p2: String) -&gt; GameState\n\nSynonyms:\nchess.setup\nchess setup","url":"chess.html#setup"},{"doc":"chess","title":"start","content":"pub fn start(state: GameState) -&gt; GameState\n\nSynonyms:\nchess.start\nchess start","url":"chess.html#start"},{"doc":"chess","title":"chess","content":"","url":"chess.html"},{"doc":"session","title":"Message","content":"pub type Message {\n  Join(String)\n  Set(GameState)\n  Close\n  GetPlayers(Subject(List(String)))\n  GetGame(Subject(Option(GameState)))\n}\n\nJoin(String)\n\n\nSet(GameState)\n\n\nClose\n\n\nGetPlayers(Subject(List(String)))\n\n\nGetGame(Subject(Option(GameState)))\n\n\nSynonyms:\nsession.Message\nsession Message","url":"session.html#Message"},{"doc":"session","title":"close","content":"pub fn close(subject: Subject(Message)) -&gt; Nil\n\nSynonyms:\nsession.close\nsession close","url":"session.html#close"},{"doc":"session","title":"get_game","content":"pub fn get_game(subject: Subject(Message)) -&gt; Option(GameState)\n\nSynonyms:\nsession.get_game\nsession get_game","url":"session.html#get_game"},{"doc":"session","title":"get_players","content":"pub fn get_players(subject: Subject(Message)) -&gt; List(String)\n\nSynonyms:\nsession.get_players\nsession get_players","url":"session.html#get_players"},{"doc":"session","title":"join","content":"pub fn join(subject: Subject(Message), name: String) -&gt; Nil\n\nSynonyms:\nsession.join\nsession join","url":"session.html#join"},{"doc":"session","title":"start","content":"pub fn start(name: String) -&gt; Result(Subject(Message), StartError)\n\nSynonyms:\nsession.start\nsession start","url":"session.html#start"},{"doc":"session","title":"start_link","content":"pub fn start_link(name: String) -&gt; Result(Pid, Dynamic)\n\nSynonyms:\nsession.start_link\nsession start_link","url":"session.html#start_link"},{"doc":"session","title":"update","content":"pub fn update(subject: Subject(Message), game: GameState) -&gt; Nil\n\nSynonyms:\nsession.update\nsession update","url":"session.html#update"},{"doc":"session","title":"session","content":"","url":"session.html"},{"doc":"crow","title":"inspect","content":"pub external fn inspect(a) -&gt; a\n\nSynonyms:\ncrow.inspect\ncrow inspect","url":"crow.html#inspect"},{"doc":"crow","title":"main","content":"pub fn main() -&gt; List(String)\n\nSynonyms:\ncrow.main\ncrow main","url":"crow.html#main"},{"doc":"crow","title":"random_float","content":"pub external fn random_float() -&gt; Float\n\nSynonyms:\ncrow.random_float\ncrow random_float","url":"crow.html#random_float"},{"doc":"crow","title":"crow","content":"","url":"crow.html"}]);